import argparse
import os
from podcast2notion.utils import get_embed
from podcast2notion.notion_helper import NotionHelper
def get_file():
    # 设置文件夹路径
    folder_path = './OUT_FOLDER'

    # 检查文件夹是否存在
    if os.path.exists(folder_path) and os.path.isdir(folder_path):
        entries = os.listdir(folder_path)
        
        file_name = entries[0] if entries else None
        return file_name
    else:
        print("OUT_FOLDER does not exist.")
        return None
    
def main():
    image_file = get_file()
    if image_file:
        image_url = f"https://raw.githubusercontent.com/{os.getenv('REPOSITORY')}/{os.getenv('REF').split('/')[-1]}/OUT_FOLDER/{image_file}"
        heatmap_url = f"https://heatmap.malinkang.com/?image={image_url}"
        if notion_helper.heatmap_block_id:
            response = notion_helper.update_heatmap(
                block_id=notion_helper.heatmap_block_id, url=heatmap_url
            )
        else:
            response = notion_helper.append_blocks(
                block_id=notion_helper.page_id, children=[get_embed(heatmap_url)]
            )
notion_helper = NotionHelper()
if __name__ == "__main__":
   main()



import json
import os
import time
from retrying import retry
import requests
from podcast2notion.notion_helper import NotionHelper
from podcast2notion import utils
from dotenv import load_dotenv
import urllib.parse

load_dotenv()


headers = {
    "accept": "application/json, text/plain, */*",
    "accept-language": "zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7",
    "content-type": "application/json",
    "origin": "https://tongyi.aliyun.com",
    "priority": "u=1, i",
    "referer": "https://tongyi.aliyun.com/efficiency/doc/transcripts/g2y8qeaoogbxnbeo?source=2",
    "sec-ch-ua": '"Not/A)Brand";v="8", "Chromium";v="126", "Google Chrome";v="126"',
    "sec-ch-ua-mobile": "?0",
    "sec-ch-ua-platform": '"macOS"',
    "sec-fetch-dest": "empty",
    "sec-fetch-mode": "cors",
    "sec-fetch-site": "same-site",
    "user-agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
    "x-b3-sampled": "1",
    "x-b3-spanid": "540e0d18e52cdf0d",
    "x-b3-traceid": "75a25320c048cde87ea3b710a65d196b",
    "x-tw-canary": "",
    "x-tw-from": "tongyi",
}


@retry(stop_max_attempt_number=3, wait_fixed=5000)
def get_dir():
    """获取文件夹"""
    url = (
        "https://qianwen.biz.aliyun.com/assistant/api/record/dir/list/get?c=tongyi-web"
    )
    response = requests.post(url, headers=headers)
    if response.ok:
        r = response.json()
        success = r.get("success")
        errorMsg = r.get("errorMsg")
        if success:
            return r.get("data")

        else:
            print(f"请求失败：{errorMsg}")
    else:
        print("请求失败：", response.status_code)


@retry(stop_max_attempt_number=3, wait_fixed=5000)
def get_record(dir_id,title):
    """获取文件夹内所有的item"""
    pageNo = 1
    pageSize = 12
    payload = {
        "dirIdStr": dir_id,
        "pageNo": pageNo,
        "pageSize": pageSize,
        "status": [20, 30, 40, 41],
        "showName":title
    }
    url = "https://qianwen.biz.aliyun.com/assistant/api/record/list?c=tongyi-web"
    response = requests.post(url, headers=headers, json=payload)
    if response.status_code == 200:
        batchRecord = response.json().get("data").get("batchRecord")
        if(len(batchRecord) > 0):
            return batchRecord[0].get("recordList")[0]


@retry(stop_max_attempt_number=3, wait_fixed=5000)
def get_note(transId):
    """暂时不支持子list和表格"""
    url = "https://tw-efficiency.biz.aliyun.com/api/doc/getTransDocEdit?c=tongyi-web"
    payload = {"action": "getTransDocEdit", "version": "1.0", "transId": transId}
    response = requests.post(url, headers=headers, json=payload)
    if response.ok:
        note = response.json().get("data").get("content")
        if note:
            data = json.loads(note)
            children = []
            for paragraph in data:
                type = "paragraph"
                skip_outer = False
                is_checked = False
                rich_text = []
                if isinstance(paragraph, list):
                    for span in paragraph:
                        if isinstance(span, dict):
                            if "list" in span:
                                type = "bulleted_list_item"
                                isOrdered = span.get("list").get("isOrdered")
                                isTaskList = span.get("list").get("isTaskList")
                                if isOrdered:
                                    type = "numbered_list_item"
                                if isTaskList:
                                    type = "to_do"
                                    is_checked = span.get("list").get("isChecked")
                        if isinstance(span, list):
                            if span[0] == "span":
                                for i in range(2, len(span)):
                                    bold = False
                                    highlight = False
                                    content = span[i][2]
                                    if "bold" in span[i][1]:
                                        bold = span[i][1].get("bold")
                                    if "highlight" in span[i][1]:
                                        highlight = True
                                    rich_text.append(get_text(content, bold, highlight))
                            if span[0] == "tag":
                                time = utils.format_milliseconds(
                                    span[1].get("metadata").get("time")
                                )
                                rich_text.append(
                                    {
                                        "type": "text",
                                        "text": {"content": time},
                                        "annotations": {
                                            "underline": True,
                                            "color": "blue",
                                        },
                                    }
                                )
                            if span[0] == "img":
                                skip_outer = True
                                url = span[1].get("src")
                                children.append(
                                    {
                                        "type": "image",
                                        "image": {
                                            "type": "external",
                                            "external": {"url": url},
                                        },
                                    }
                                )
                if skip_outer:
                    continue
                child = {
                    "type": type,
                    type: {"rich_text": rich_text, "color": "default"},
                }
                if type == "to_do":
                    child[type]["checked"] = is_checked
                children.append(child)
            return children
    else:
        print("请求失败")


def get_text(content, bold=False, highlight=False):
    text = {"type": "text", "text": {"content": content}, "annotations": {"bold": bold}}
    if highlight:
        text["annotations"]["color"] = "red_background"
    return text


@retry(stop_max_attempt_number=3, wait_fixed=5000)
def get_all_lab_info(transId):
    url = "https://tw-efficiency.biz.aliyun.com/api/lab/getAllLabInfo?c=tongyi-web"
    payload = {
        "action": "getAllLabInfo",
        "content": ["labInfo", "labSummaryInfo"],
        "transId": transId,
    }
    response = requests.post(url, headers=headers, json=payload)
    mindmap = None
    children = []
    if response.status_code == 200:
        data = response.json().get("data")
        labInfo = data.get("labCardsMap").get("labInfo")
        labInfo.extend(data.get("labCardsMap").get("labSummaryInfo"))
        for i in labInfo:
            name = i.get("basicInfo").get("name")
            if name == "qa问答":
                children.append(utils.get_heading(2, "问题回顾"))
            if name == "议程":
                children.append(utils.get_heading(2, "章节速览"))
            for content in i.get("contents", []):
                for contentValue in content.get("contentValues"):
                    if name == "全文摘要":
                        value = contentValue.get("value")
                        children.append(utils.get_heading(3, "全文摘要"))
                        children.append(utils.get_callout(value, {"emoji": "💡"}))
                    if name == "思维导图":
                        mindmap = contentValue.get("json")
                    if name == "议程":
                        title = f"{utils.format_milliseconds(contentValue.get('time'))} {contentValue.get('value')}"
                        children.append(utils.get_heading(3, title))
                        summary = contentValue.get("summary")
                        if summary:
                            children.append(utils.get_callout(summary, {"emoji": "💡"}))
                    if name == "qa问答" and contentValue.get("extensions"):
                        title = contentValue.get("title")
                        value = contentValue.get("value")
                        if contentValue.get("extensions")[0].get(
                            "sentenceInfoOfAnswer"
                        ):
                            beginTime = (
                                contentValue.get("extensions")[0]
                                .get("sentenceInfoOfAnswer")[0]
                                .get("beginTime")
                            )
                            if beginTime:
                                beginTime = utils.format_milliseconds(beginTime)
                                title = f"{beginTime} {title}"
                        children.append(utils.get_heading(3, title))
                        children.append(utils.get_callout(value, {"emoji": "💡"}))
        return (children, mindmap)
    else:
        print("请求脑图失败：", response.status_code)


def insert_mindmap(mindmap,title,episode,children):
    mindmap_page_id = None
    if mindmap:
        mindmap_page_id = create_mindmap(title, episode.get("icon"))
        start_time = time.time()
        print(f"开始插入思维导图")
        with open('mindmap.json', 'w', encoding='utf-8') as f:
            json.dump(mindmap, f, ensure_ascii=False, indent=4)
        mindmap_root_id = (
            notion_helper.append_blocks(
                block_id=mindmap_page_id,
                children=[utils.get_heading(1,mindmap.get("content"))],
            )
            .get("results")[0]
            .get("id")
        )
        insert_mindmap_to_notion(mindmap_page_id,mindmap_root_id, mindmap.get("children"),2)
        update_mindmap(mindmap_page_id)
        end = time.time()
        print(f"插入思维导图结束，耗时{round(end-start_time)}秒")
        mindmap_url = f"https://mindmap.malinkang.com/markmap/{mindmap_page_id.replace('-','')}?token={os.getenv('NOTION_TOKEN')}"
        children.append(utils.get_heading(2, "思维导图"))
        children.append({"type": "embed", "embed": {"url": mindmap_url}})
    return mindmap_page_id


def insert_mindmap_to_notion(page_id,block_id, children,level):
    """将思维导图插入Notion中"""
    blocks = [ utils.get_heading(level,block.get("content")) if(level < 4) else utils.get_bulleted_list_item(block.get("content"))for block in children]
    if(level < 5):
        results = notion_helper.append_blocks_after(block_id=page_id,after=block_id,children=blocks).get("results")
    else:
        results = notion_helper.append_blocks(block_id=block_id,children=blocks).get("results")
    for index, child in enumerate(children):
        if child.get("children"):
            insert_mindmap_to_notion(page_id,results[index].get("id"), child.get("children"),level+1)


def check_mindmap(title):
    """检查是否已经插入过"""
    filter = {"property": "标题", "title": {"equals": title}}
    response = notion_helper.query(
        database_id=notion_helper.mindmap_database_id, filter=filter
    )
    if len(response["results"]) > 0:
        return response["results"][0]


def create_mindmap(title, icon):
    result = check_mindmap(title)
    if result:
        status = utils.get_property_value(result.get("properties").get("状态"))
        if status == "Done":
            return result.get("id")
        else:
            notion_helper.delete_block(result.get("id"))
    parent = {
        "database_id": notion_helper.mindmap_database_id,
        "type": "database_id",
    }
    properties = {
        "标题": {"title": [{"type": "text", "text": {"content": title}}]},
        "状态": {"status": {"name": "In progress"}},
    }

    mindmap_page_id = notion_helper.create_page(
        parent=parent,
        properties=properties,
        icon=icon,
    ).get("id")
    return mindmap_page_id


def update_mindmap(page_id):
    properties = {
        "状态": {"status": {"name": "Done"}},
    }

    notion_helper.update_page(
        page_id=page_id,
        properties=properties,
    )


@retry(stop_max_attempt_number=3, wait_fixed=5000)
def get_trans_result(transId):
    payload = {
        "action": "getTransResult",
        "version": "1.0",
        "transId": transId,
    }
    url = "https://tw-efficiency.biz.aliyun.com/api/trans/getTransResult?c=tongyi-web"
    response = requests.post(url, headers=headers, json=payload)
    if response.ok:
        response_data = response.json()
        user_dict = {}
        if response_data.get("data").get("tag").get("identify"):
            user_info = json.loads(
                response_data.get("data").get("tag").get("identify")
            ).get("user_info")
            for key, value in user_info.items():
                user_dict[key] = value.get("name")
        children = []
        for i in json.loads(response_data.get("data").get("result")).get("pg",[]):
            content = ""
            name = ""
            uid = i.get("ui")
            avatar = None
            if uid in user_dict:
                name = user_dict.get(uid)
                avatar = get_author_avatar(name)
            else:
                name = f"发言人{uid}"
            if avatar is None:
                avatar = "https://www.notion.so/icons/user_gray.svg"
            title = f'{name} {utils.format_milliseconds(i.get("sc")[0].get("bt"))}'
            children.append(utils.get_heading(3, title))
            for j in i.get("sc"):
                content += j.get("tc")
            children.append(utils.get_callout(content, utils.get_icon(avatar)))
        return children
    else:
        print("请求失败：", response.status_code)
        return None


author_cache = {}


@retry(stop_max_attempt_number=3, wait_fixed=5000)
def get_author_avatar(author):
    if author in author_cache:
        return author_cache[author]
    filter = {"property": "标题", "title": {"equals": author}}
    r = notion_helper.query(database_id=notion_helper.author_database_id, filter=filter)
    if len(r.get("results")) > 0:
        avatar = r.get("results")[0].get("icon").get("external").get("url")
        author_cache[author] = avatar
        return avatar






cache = {}


def get_podcast(ids):
    podcast_page_id = ids[0].get("id")
    if id not in cache:
        podcast_properties = notion_helper.client.pages.retrieve(podcast_page_id).get(
            "properties"
        )
        cache[podcast_page_id] = podcast_properties
    return cache.get(podcast_page_id)


def get_dir_id_by_name(dir_name,all_dirs):
    dir = list(filter(lambda x: x.get("dir").get("dirName") == dir_name, all_dirs))
    if dir:
        return dir[0].get("dir").get("id")

notion_helper = NotionHelper()

def main():
    headers["cookie"] = os.getenv("COOKIE").strip()
    f = {
        "and": [
            {"property": "通义链接", "url": {"is_not_empty": True}},
            {"property": "语音转文字状态", "status": {"does_not_equal": "Done"}},
            {"property": "Podcast", "relation": {"is_not_empty": True}},
        ]
    }
    sorts = [
        {"property": "日期", "direction": "descending"}
    ]
    episodes = notion_helper.query_all_by_filter(
        notion_helper.episode_database_id, filter=f, sorts=sorts
    )
    podcasts = {}
    pids = []
    for episode in episodes:
        episode_properties = episode.get("properties")
        podcast = utils.get_property_value(episode_properties.get("Podcast"))
        podcast_properties = get_podcast(podcast)
        podcast_title = utils.get_property_value(podcast_properties.get("播客"))
        pid = utils.get_property_value(podcast_properties.get("Pid"))
        if podcast_title not in podcasts:
            podcasts[podcast_title] = {"id":pid, "episodes": []}
            pids.append({"id":pid,"title":podcast_title})
        podcasts[podcast_title].get("episodes").append(episode)
    all_dirs = get_dir()
    for key, value in podcasts.items():
        dir_id = get_dir_id_by_name(key,all_dirs)
        if dir_id:
            for episode in value.get("episodes"):
                episode_properties = episode.get("properties")
                title = utils.get_property_value(episode_properties.get("标题"))
                children = []
                # 20 正在转 30是成功 40是失败
                record = get_record(dir_id,title)
                if record:
                    if record.get("recordStatus") != 30:
                        continue
                    episode_page_id = episode.get("id")
                    audio_url = utils.get_property_value(episode_properties.get("音频"))
                    transId = record.get("genRecordId")
                    cover = episode.get("cover").get("external").get("url")
                    children.append(utils.get_heading(2, "音频"))
                    if audio_url and title and cover and podcast_title:
                        player_url = f"https://notion-music.malinkang.com/player?url={urllib.parse.quote(audio_url)}&name={urllib.parse.quote(title)}&cover={urllib.parse.quote(cover)}&artist={urllib.parse.quote(podcast_title)}"
                        children.append({"type": "embed", "embed": {"url": player_url}})
                    print(f"开始获取《{title}》的数据")
                    info, mindmap = get_all_lab_info(transId)
                    mindmap_page_id = insert_mindmap(mindmap,title,episode,children)
                    if info:
                        children.extend(info)
                    trans = get_trans_result(transId)
                    if trans:
                        children.append(utils.get_heading(2, "语音转文字"))
                        children.extend(trans)
                    note = get_note(transId)
                    if note:
                        children.append(utils.get_heading(2, "笔记"))
                        children.extend(note)
                    start_time = time.time()
                    print(f"开始插入其他数据")
                    for i in range(0, len(children) // 100 + 1):
                        notion_helper.append_blocks(
                            block_id=episode_page_id,
                            children=children[i * 100 : (i + 1) * 100],
                        )
                    end = time.time()
                    print(f"插入其他数据结束，耗时{round(end-start_time)}秒")
                    properties = {"语音转文字状态": {"status": {"name": "Done"}}}
                    if mindmap_page_id:
                        properties["思维导图"] = {"relation": [{"id": mindmap_page_id}]}
                    notion_helper.update_page(
                        page_id=episode_page_id,
                        properties=properties,
                    )

if __name__ == "__main__":
    main()


import logging
import os
import re
import time

from notion_client import Client
from retrying import retry
from datetime import timedelta
from dotenv import load_dotenv
from podcast2notion import utils
load_dotenv()
from podcast2notion.utils import (
    format_date,
    get_date,
    get_first_and_last_day_of_month,
    get_first_and_last_day_of_week,
    get_first_and_last_day_of_year,
    get_icon,
    get_relation,
    get_title,
)

TAG_ICON_URL = "https://www.notion.so/icons/tag_gray.svg"
USER_ICON_URL = "https://www.notion.so/icons/user-circle-filled_gray.svg"
TARGET_ICON_URL = "https://www.notion.so/icons/target_red.svg"
BOOKMARK_ICON_URL = "https://www.notion.so/icons/bookmark_gray.svg"


class NotionHelper:
    database_name_dict = {
        "PODCAST_DATABASE_NAME": "Podcast",
        "EPISODE_DATABASE_NAME": "Episode",
        "ALL_DATABASE_NAME": "全部",
        "AUTHOR_DATABASE_NAME": "Author",
        "MINDMAP_DATABASE_NAME": "思维导图",
    }
    database_id_dict = {}
    heatmap_block_id = None
    property_dict = {}

    def __init__(self):
        self.client = Client(auth=os.getenv("NOTION_TOKEN").strip(), log_level=logging.ERROR)
        self.__cache = {}
        self.page_id = self.extract_page_id(os.getenv("NOTION_PAGE").strip())
        self.search_database(self.page_id)
        for key in self.database_name_dict.keys():
            if os.getenv(key) != None and os.getenv(key) != "":
                self.database_name_dict[key] = os.getenv(key)
        self.episode_database_id = self.database_id_dict.get(
            self.database_name_dict.get("EPISODE_DATABASE_NAME")
        )
        self.podcast_database_id = self.database_id_dict.get(
            self.database_name_dict.get("PODCAST_DATABASE_NAME")
        )
        self.author_database_id = self.database_id_dict.get(
            self.database_name_dict.get("AUTHOR_DATABASE_NAME")
        )
        if self.author_database_id is None:
            self.author_database_id = self.database_id_dict.get("作者")
        self.all_database_id = self.database_id_dict.get(
            self.database_name_dict.get("ALL_DATABASE_NAME")
        )
        self.mindmap_database_id = self.database_id_dict.get(
            self.database_name_dict.get("MINDMAP_DATABASE_NAME")
        )
        r = self.client.databases.retrieve(database_id=self.episode_database_id)
        for key, value in r.get("properties").items():
            self.property_dict[key] = value
        self.day_database_id = self.get_relation_database_id(
            self.property_dict.get("日")
        )
        self.week_database_id = self.get_relation_database_id(
            self.property_dict.get("周")
        )
        self.month_database_id = self.get_relation_database_id(
            self.property_dict.get("月")
        )
        self.year_database_id = self.get_relation_database_id(
            self.property_dict.get("年")
        )
        self.all_database_id = self.get_relation_database_id(
            self.property_dict.get("全部")
        )
        if self.day_database_id:
            self.write_database_id(self.day_database_id)
        if self.podcast_database_id:
            self.update_database(self.podcast_database_id)
        if self.episode_database_id:
            self.update_database(self.episode_database_id)
    @retry(stop_max_attempt_number=3, wait_fixed=5000)
    def update_database(self,database_id):
        """更新数据库"""
        response = self.client.databases.retrieve(database_id=database_id)
        id = response.get("id")
        properties = response.get("properties")
        update_properties = {}
        if (
            properties.get("通义链接") is None
            or properties.get("通义链接").get("type") != "url"
        ):
            update_properties["通义链接"] = {"url": {}}
        if len(update_properties) > 0:
            self.client.databases.update(database_id=id, properties=update_properties)
    def get_relation_database_id(self, property):
        return property.get("relation").get("database_id")

    def write_database_id(self, database_id):
        env_file = os.getenv('GITHUB_ENV')
        # 将值写入环境文件
        with open(env_file, "a") as file:
            file.write(f"DATABASE_ID={database_id}\n")
    def extract_page_id(self, notion_url):
        # 正则表达式匹配 32 个字符的 Notion page_id
        match = re.search(
            r"([a-f0-9]{32}|[a-f0-9]{8}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{12})",
            notion_url,
        )
        if match:
            return match.group(0)
        else:
            raise Exception(f"获取NotionID失败，请检查输入的Url是否正确")

    def search_database(self, block_id):
        try:
            children = self.client.blocks.children.list(block_id=block_id)["results"]
            # 遍历子块
            for child in children:
                # 检查子块的类型
                if child["type"] == "child_database":
                    self.database_id_dict[child.get("child_database").get("title")] = (
                        child.get("id")
                    )
                elif child["type"] == "embed" and child.get("embed").get("url"):
                    if child.get("embed").get("url").startswith("https://heatmap.malinkang.com/"):
                        self.heatmap_block_id = child.get("id")
                # 如果子块有子块，递归调用函数
                if "has_children" in child and child["has_children"]:
                    self.search_database(child["id"])
        except Exception as e:
            print(f"搜索数据库时发生异常: {e}")

    @retry(stop_max_attempt_number=3, wait_fixed=5000)
    def update_image_block_link(self, block_id, new_image_url):
        # 更新 image block 的链接
        self.client.blocks.update(
            block_id=block_id, image={"external": {"url": new_image_url}}
        )

    def get_week_relation_id(self, date):
        year = date.isocalendar().year
        week = date.isocalendar().week
        week = f"{year}年第{week}周"
        start, end = get_first_and_last_day_of_week(date)
        properties = {"日期": get_date(format_date(start), format_date(end))}
        return self.get_relation_id(
            week, self.week_database_id, self.get_date_icon(date, "week"), properties
        )

    def get_month_relation_id(self, date):
        month = date.strftime("%Y年%m月")
        start, end = get_first_and_last_day_of_month(date)
        year = self.get_year_relation_id(date)
        properties = {
            "日期": get_date(format_date(start), format_date(end)),
            "年": get_relation([year]),
        }
        return self.get_relation_id(
            month, self.month_database_id, self.get_date_icon(date, "month"), properties
        )

    def get_year_relation_id(self, date):
        year = date.strftime("%Y")
        start, end = get_first_and_last_day_of_year(date)
        properties = {"日期": get_date(format_date(start), format_date(end))}
        return self.get_relation_id(
            year, self.year_database_id, self.get_date_icon(date, "year"), properties
        )

    def get_date_icon(self, date, type):
        return f"https://notion-icon.malinkang.com/?type={type}&date={date.strftime('%Y-%m-%d')}"

    def get_day_relation_id(self, date, properties={}):
        new_date = date.replace(hour=0, minute=0, second=0, microsecond=0)
        day = new_date.strftime("%Y年%m月%d日")
        properties["日期"] = get_date(format_date(date))
        return self.get_relation_id(
            day, self.day_database_id, self.get_date_icon(date, "day"), properties
        )

    @retry(stop_max_attempt_number=3, wait_fixed=5000)
    def get_relation_id(self, name, id, icon, properties={}):
        key = f"{id}{name}"
        if key in self.__cache:
            return self.__cache.get(key)
        filter = {"property": "标题", "title": {"equals": name}}
        response = self.client.databases.query(database_id=id, filter=filter)
        if len(response.get("results")) == 0:
            parent = {"database_id": id, "type": "database_id"}
            properties["标题"] = get_title(name)
            page_id = self.client.pages.create(
                parent=parent, properties=properties, icon=get_icon(icon)
            ).get("id")
        else:
            page_id = response.get("results")[0].get("id")
        self.__cache[key] = page_id
        return page_id

    @retry(stop_max_attempt_number=3, wait_fixed=5000)
    def update_book_page(self, page_id, properties):
        return self.client.pages.update(page_id=page_id, properties=properties)

    @retry(stop_max_attempt_number=3, wait_fixed=5000)
    def update_page(self, page_id, properties):
        return self.client.pages.update(page_id=page_id, properties=properties)

    @retry(stop_max_attempt_number=3, wait_fixed=5000)
    def create_page(self, parent, properties, icon):
        return self.client.pages.create(
            parent=parent, properties=properties, icon=icon, cover=icon
        )

    @retry(stop_max_attempt_number=3, wait_fixed=5000)
    def query(self, **kwargs):
        kwargs = {k: v for k, v in kwargs.items() if v}
        return self.client.databases.query(**kwargs)

    @retry(stop_max_attempt_number=3, wait_fixed=5000)
    def get_block_children(self, id):
        response = self.client.blocks.children.list(id)
        return response.get("results")

    @retry(stop_max_attempt_number=3, wait_fixed=5000)
    def append_blocks(self, block_id, children):
        return self.client.blocks.children.append(block_id=block_id, children=children)

    @retry(stop_max_attempt_number=3, wait_fixed=5000)
    def append_blocks_after(self, block_id, children, after):
        return self.client.blocks.children.append(
            block_id=block_id, children=children, after=after
        )

    @retry(stop_max_attempt_number=3, wait_fixed=5000)
    def delete_block(self, block_id):
        return self.client.blocks.delete(block_id=block_id)

    @retry(stop_max_attempt_number=3, wait_fixed=5000)
    def query_all_by_filter(self, database_id, filter,sorts):
        results = []
        has_more = True
        start_cursor = None
        while has_more:
            response = self.client.databases.query(
                database_id=database_id,
                filter=filter,
                sorts=sorts,
                start_cursor=start_cursor,
                page_size=100,
            )
            start_cursor = response.get("next_cursor")
            has_more = response.get("has_more")
            results.extend(response.get("results"))
        return results
    
    @retry(stop_max_attempt_number=3, wait_fixed=5000)
    def get_all_podcast(self):
        results = self.query_all(self.podcast_database_id)
        podcast_dict = {}
        for result in results:
            pid = utils.get_property_value(result.get("properties").get("Pid"))
            podcast_dict[pid] = {
                "page_id": result.get("id"),
                "最后更新时间": utils.get_property_value(
                    result.get("properties").get("最后更新时间")
                ),
                "收听时长": utils.get_property_value(
                    result.get("properties").get("收听时长")
                ),
                "通义链接": utils.get_property_value(result.get("properties").get("通义链接"))
                
            }
        return podcast_dict
    
    @retry(stop_max_attempt_number=3, wait_fixed=5000)
    def get_all_episode(self):
        results = self.query_all(self.episode_database_id)
        episode_dict = {}
        for result in results:
            eid = utils.get_property_value(result.get("properties").get("Eid"))
            episode_dict[eid] = {
                "page_id": result.get("id"),
                "状态": utils.get_property_value(
                    result.get("properties").get("状态")
                ),
                "喜欢": utils.get_property_value(
                    result.get("properties").get("喜欢")
                ),          
                "收听进度": utils.get_property_value(
                    result.get("properties").get("收听进度")
                ),          
                "语音转文字状态": utils.get_property_value(
                    result.get("properties").get("语音转文字状态")
                ),
                "通义链接": utils.get_property_value(
                    result.get("properties").get("通义链接")
                ),
                "日期": utils.get_property_value(
                    result.get("properties").get("日期")
                )
            }
        return episode_dict

    @retry(stop_max_attempt_number=3, wait_fixed=5000)
    def query_all(self, database_id):
        """获取database中所有的数据"""
        results = []
        has_more = True
        start_cursor = None
        while has_more:
            response = self.client.databases.query(
                database_id=database_id,
                start_cursor=start_cursor,
                page_size=100,
            )
            start_cursor = response.get("next_cursor")
            has_more = response.get("has_more")
            results.extend(response.get("results"))
        return results

    def get_all_relation(self, properties):
        properties["全部"] = get_relation(
            [
                self.get_relation_id("全部", self.all_database_id, TARGET_ICON_URL),
            ]
        )

    def get_date_relation(self, properties, date):
        properties["年"] = get_relation(
            [
                self.get_year_relation_id(date),
            ]
        )
        properties["月"] = get_relation(
            [
                self.get_month_relation_id(date),
            ]
        )
        properties["周"] = get_relation(
            [
                self.get_week_relation_id(date),
            ]
        )
        properties["日"] = get_relation(
            [
                self.get_day_relation_id(date),
            ]
        )
    @retry(stop_max_attempt_number=3, wait_fixed=5000)
    def update_heatmap(self, block_id, url):
        # 更新 image block 的链接
        return self.client.blocks.update(block_id=block_id, embed={"url": url})



import os
import time
import pendulum
from retrying import retry
import requests
from podcast2notion.notion_helper import NotionHelper
from podcast2notion import utils
from dotenv import load_dotenv

load_dotenv()

from podcast2notion.config import (
    movie_properties_type_dict,
    book_properties_type_dict,
    TAG_ICON_URL,
    TZ,
)
from podcast2notion.utils import get_icon


def generate_activation_code():
    """生成并设置激活码"""
    url = "https://podcast.notionhub.app/generate-activation-code"
    payload = {
        "access_token": os.getenv("NOTION_TOKEN"),
        "podcast": os.getenv("REFRESH_TOKEN"),
        "tongyi": os.getenv("TONGYI"),
        "duplicated_template_id": os.getenv("NOTION_PAGE"),
    }
    try:
        response = requests.post(url, json=payload)
        response.raise_for_status()
        activation_code = response.json().get("activation_code")
        if activation_code:
            os.environ["ACTIVATION_CODE"] = activation_code
            print("激活码生成并设置成功")
        else:
            print("获取激活码失败")
    except requests.exceptions.RequestException as e:
        print(f"调用激活码接口失败: {e}")


headers = {
    "host": "api.xiaoyuzhoufm.com",
    "applicationid": "app.podcast.cosmos",
    "x-jike-refresh-token": os.getenv("REFRESH_TOKEN").strip(),
    "x-jike-device-id": "5070e349-ba04-4c7b-a32e-13eb0fed01e7",
}

tongyi_headers = {
    "accept": "application/json, text/plain, */*",
    "accept-language": "zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7",
    "content-type": "application/json",
    "origin": "https://tongyi.aliyun.com",
    "priority": "u=1, i",
    "referer": "https://tongyi.aliyun.com/efficiency/doc/transcripts/g2y8qeaoogbxnbeo?source=2",
    "sec-ch-ua": '"Not/A)Brand";v="8", "Chromium";v="126", "Google Chrome";v="126"',
    "sec-ch-ua-mobile": "?0",
    "sec-ch-ua-platform": '"macOS"',
    "sec-fetch-dest": "empty",
    "sec-fetch-mode": "cors",
    "sec-fetch-site": "same-site",
    "user-agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
    "x-b3-sampled": "1",
    "x-b3-spanid": "540e0d18e52cdf0d",
    "x-b3-traceid": "75a25320c048cde87ea3b710a65d196b",
    "x-tw-canary": "",
    "cookie": os.getenv("COOKIE").strip(),
    "x-tw-from": "tongyi",
}



@retry(stop_max_attempt_number=3, wait_fixed=5000)
def refresh_token():
    url = "https://api.xiaoyuzhoufm.com/app_auth_tokens.refresh"
    resp = requests.post(url, headers=headers)
    if resp.ok:
        token = resp.json().get("x-jike-access-token")
        headers["x-jike-access-token"] = token


@retry(stop_max_attempt_number=3, wait_fixed=5000)
def get_podcast():
    results = []
    url = "https://api.xiaoyuzhoufm.com/v1/subscription/list"
    data = {
        "limit": 25,
        "sortBy": "subscribedAt",
        "sortOrder": "desc",
    }
    loadMoreKey = ""
    while loadMoreKey is not None:
        if loadMoreKey:
            data["loadMoreKey"] = loadMoreKey
        resp = requests.post(url, json=data, headers=headers)
        if resp.ok:
            loadMoreKey = resp.json().get("loadMoreKey")
            results.extend(resp.json().get("data"))
        else:
            refresh_token()
            raise Exception(f"Error {data} {resp.text}")
    return results

@retry(stop_max_attempt_number=3, wait_fixed=5000)
def get_mileage(rank="TOTAL"):
    results = []
    url = "https://api.xiaoyuzhoufm.com/v1/mileage/list"
    data = {"rank": rank}
    loadMoreKey = ""
    while loadMoreKey is not None:
        if loadMoreKey:
            data["loadMoreKey"] = loadMoreKey
        resp = requests.post(url, json=data, headers=headers)
        if resp.ok:
            loadMoreKey = resp.json().get("loadMoreKey")
            for item in resp.json().get("data"):
                podcast = item.get("podcast")
                podcast["playedSeconds"] = item.get("playedSeconds", 0)
                results.append(podcast)
        else:
            refresh_token()
            raise Exception(f"Error {data} {resp.text}")
    return results


@retry(stop_max_attempt_number=3, wait_fixed=5000)
def get_episode(pid, timestamp):
    results = []
    url = "https://api.xiaoyuzhoufm.com/v1/episode/list"
    data = {
        "limit": 25,
        "pid": pid,
    }
    loadMoreKey = ""
    while loadMoreKey is not None:
        if loadMoreKey:
            data["loadMoreKey"] = loadMoreKey
        resp = requests.post(url, json=data, headers=headers)
        if resp.ok:
            loadMoreKey = resp.json().get("loadMoreKey")
            d = resp.json().get("data")
            for item in d:
                pubDate = pendulum.parse(item.get("pubDate")).in_tz("UTC").int_timestamp
                if pubDate <= timestamp:
                    return results
                item["pubDate"] = pubDate
                results.append(item)
        else:
            refresh_token()
            raise Exception(f"Error {data} {resp.text}")
    return results


@retry(stop_max_attempt_number=3, wait_fixed=5000)
def get_history():
    results = []
    url = "https://api.xiaoyuzhoufm.com/v1/episode-played/list-history"
    data = {
        "limit": 25,
    }
    loadMoreKey = ""
    while loadMoreKey is not None:
        if loadMoreKey:
            data["loadMoreKey"] = loadMoreKey
        resp = requests.post(url, json=data, headers=headers)
        if resp.ok:
            loadMoreKey = resp.json().get("loadMoreKey")
            d = resp.json().get("data")
            for item in d:
                episode = item.get("episode")
                pubDate = (
                    pendulum.parse(episode.get("pubDate")).in_tz("UTC").int_timestamp
                )
                episode["pubDate"] = pubDate
                results.append(episode)
        else:
            refresh_token()
            raise Exception(f"Error {data} {resp.text}")
    return results


def merge_podcast(list1, list2):
    results = []
    results.extend(list1)
    d = {x.get("pid"): x for x in list1}
    for item in list2:
        if item.get("pid") not in d:
            results.append(item)
    return results



def insert_podcast(dir_dict):
    list1 = get_mileage()
    list2 = get_podcast()
    results = merge_podcast(list1, list2)
    notion_podcasts = notion_helper.get_all_podcast()
    dict = {}
    for index, result in enumerate(results):
        podcast = {}
        title =result.get("title")
        dir_id = dir_dict.get(title)
        if dir_id is None:
            dir_id = create_dir(title)
            dir_dict[title] = dir_id
        podcast["播客"] = title
        podcast["通义链接"] = f"https://tongyi.aliyun.com/efficiency/folders/{dir_id}"
        podcast["Brief"] = result.get("brief")
        pid = result.get("pid")
        podcast["Pid"] = pid
        podcast["收听时长"] = result.get("playedSeconds", 0)
        podcast["Description"] = result.get("description")
        podcast["链接"] = f"https://www.xiaoyuzhoufm.com/podcast/{result.get('pid')}"
        if result.get("latestEpisodePubDate"):
            podcast["最后更新时间"] = (
                pendulum.parse(result.get("latestEpisodePubDate"))
                .replace(second=0)
                .in_tz("UTC")
                .int_timestamp
            )
        cover = result.get("image").get("picUrl")
        page_id = None
        if pid in notion_podcasts:
            old_podcast = notion_podcasts.get(pid)
            page_id = old_podcast.get("page_id")
            dict[pid] = (page_id, cover,title)
            if (
                old_podcast.get("最后更新时间") == podcast.get("最后更新时间")
                and old_podcast.get("收听时长") == podcast.get("收听时长")
                and old_podcast.get("通义链接") == podcast.get("通义链接")
            ):
                continue
        print(
            f"正在同步 = {result.get('title')}，共{len(results)}个播客，当前是第{index+1}个"
        )
        podcast["全部"] = [
            notion_helper.get_relation_id(
                "全部", notion_helper.all_database_id, TAG_ICON_URL
            )
        ]
        podcast["作者"] = [
            notion_helper.get_relation_id(
                x.get("nickname"),
                notion_helper.author_database_id,
                x.get("avatar").get("picture").get("picUrl"),
            )
            for x in result.get("podcasters")
        ]
        properties = utils.get_properties(podcast, movie_properties_type_dict)
        parent = {
            "database_id": notion_helper.podcast_database_id,
            "type": "database_id",
        }
        if page_id:
            notion_helper.update_page(page_id=page_id, properties=properties)
        else:
            page_id = notion_helper.create_page(
                parent=parent, properties=properties, icon=get_icon(cover)
            ).get("id")
        dict[pid] = (page_id, cover,title)
    return dict


def get_monthly_wrapped(year, month, id):
    url = "https://api.xiaoyuzhoufm.com/v1/monthly-wrapped/get"
    data = {"uid": get_profile(), "year": year, "month": month}
    resp = requests.get(url, params=data, headers=headers)
    playedDays = 0
    playedSeconds = 0
    if resp.ok:
        data = resp.json().get("data")
        if data:
            playedDays = data.get("playedDays")
            playedSeconds = data.get("playedSeconds")
        properties = {
            "收听时长": {"number": playedSeconds},
            "收听天数": {"number": playedDays},
        }
        notion_helper.update_page(id, properties)


def get_month_from_notion():
    filter = {
        "and": [
            {"property": "收听时长", "number": {"equals": 0}},
            {
                "property": "日期",
                "date": {"before": pendulum.now(tz=TZ).replace(day=1).to_date_string()},
            },
        ]
    }
    return notion_helper.query(
        database_id=notion_helper.month_database_id, filter=filter
    )


def update_month_data():
    for result in get_month_from_notion().get("results"):
        title = utils.get_property_value(result.get("properties").get("标题"))
        if not title:
            continue
        id = result.get("id")
        year = int(title[0:4])
        month = int(title[5 : title.index("月")])
        get_monthly_wrapped(year, month, id)


def insert_episode(episodes, d,dir_dict):
    episodes.sort(key=lambda x: x["pubDate"])
    notion_episodes = notion_helper.get_all_episode()
    for index, result in enumerate(episodes):
        pid = result.get("pid")
        if pid not in d:
            continue
        page_id = None
        eid = result.get("eid")
        episode = {}
        episode["标题"] = result.get("title")
        episode["Description"] = result.get("description")
        episode["时间戳"] = result.get("pubDate")
        episode["发布时间"] = result.get("pubDate")
        episode["音频"] = result.get("media").get("source").get("url")
        episode["Eid"] = eid
        episode["时长"] = result.get("duration")
        episode["喜欢"] = result.get("isPicked")
        episode["收听进度"] = result.get("progress")
        episode["Podcast"] = [d.get(pid)[0]]
        episode["链接"] = f"hhttps://www.xiaoyuzhoufm.com/episode/{result.get('eid')}"
        status = "未听"
        if result.get("isFinished"):
            episode["收听进度"] = result.get("duration")
            status = "听过"
        elif result.get("isPlayed"):
            status = "在听"
        episode["状态"] = status
        if result.get("playedAt"):
            episode["日期"] = (
                pendulum.parse(result.get("playedAt"))
                .replace(second=0)
                .in_tz("UTC")
                .int_timestamp
            )
        dir_name = d.get(pid)[2]
        page_id = None
        if eid in notion_episodes:
            old_episode = notion_episodes.get(eid)
            # 如果是听过将老的日期赋值为老的日期
            if old_episode.get("状态") == "听过":
                episode["日期"] = old_episode.get("日期")
            # 如果语音转文字状态不为Done，并且通义链接为空，则提交转写
            if old_episode.get("语音转文字状态")!= "Done" and old_episode.get("通义链接") is None:
                episode["通义链接"] = getTongYiUrl(dir_name,episode.get("标题"),episode.get("音频"))
            # 如果通义链接不为空，则赋值
            elif old_episode.get("通义链接") is not None:
                episode["通义链接"] = old_episode.get("通义链接")
            if (
                old_episode.get("状态") == episode.get("状态")
                and old_episode.get("喜欢") == episode.get("喜欢")
                and old_episode.get("收听进度") == episode.get("收听进度")
                and old_episode.get("日期") == episode.get("日期")
                and old_episode.get("通义链接") == episode.get("通义链接")
            ):
                continue
            page_id = old_episode.get("page_id")
        else:
            episode["通义链接"] =  getTongYiUrl(dir_dict,dir_name,episode.get("标题"),episode.get("音频"))
        print(
            f"正在同步 = {result.get('title')}，共{len(episodes)}个Episode，当前是第{index+1}个"
        )
        episode["全部"] = [
            notion_helper.get_relation_id(
                "全部", notion_helper.all_database_id, TAG_ICON_URL
            )
        ]
        properties = utils.get_properties(episode, book_properties_type_dict)
        notion_helper.get_all_relation(properties=properties)
        if episode.get("日期"):
            notion_helper.get_date_relation(
                properties=properties,
                date=pendulum.from_timestamp(episode.get("日期"), tz="Asia/Shanghai"),
            )
        parent = {
            "database_id": notion_helper.episode_database_id,
            "type": "database_id",
        }
        if page_id:
            notion_helper.update_page(page_id=page_id, properties=properties)
        else:
            notion_helper.create_page(
                parent=parent, properties=properties, icon=get_icon(d.get(pid)[1])
            )


def get_progress(eids):
    """获取播放进度"""
    url = "https://api.xiaoyuzhoufm.com/v1/playback-progress/list"
    data = {"eids": eids}
    resp = requests.post(url, json=data, headers=headers)
    if resp.ok:
        return resp.json().get("data")


def get_profile():
    url = "https://api.xiaoyuzhoufm.com/v1/profile/get"
    resp = requests.get(url, headers=headers)
    if resp.ok:
        return resp.json().get("data").get("uid")

def getTongYiUrl(dir_dict,dir_name,title,url):
    dir_id = dir_dict.get(dir_name)
    if dir_id is None:
        dir_id = create_dir(dir_name)
        dir_dict[dir_name] = dir_id
        print(f"创建文件夹${dir_name}成功")
    if dir_id:
        task_id = parseNetSourceUrl(url)
        if task_id:
            files = queryNetSourceParse(task_id,dir_id,title)
            if files:
                id = start(dir_id,files)
                return f"https://tongyi.aliyun.com/efficiency/doc/transcripts/{id}"

@retry(stop_max_attempt_number=3, wait_fixed=5000)
def create_dir(name):
    """创建文件夹，支持创建重名文件夹"""
    payload = {"dirName": name, "parentIdStr": -1}
    url = "https://qianwen.biz.aliyun.com/assistant/api/record/dir/add?c=tongyi-web"
    r = requests.post(url, headers=tongyi_headers, json=payload)
    if r.ok:
        return r.json().get("data").get("focusDir").get("idStr")

def parseNetSourceUrl(url):
    print("start parse url")
    payload = {"action": "parseNetSourceUrl", "version": "1.0", "url": url}
    url = (
        "https://tw-efficiency.biz.aliyun.com/api/trans/parseNetSourceUrl?c=tongyi-web"
    )
    r = requests.post(url, headers=tongyi_headers, json=payload)
    if r.ok:
        data = r.json()
        success = data.get("success")
        print(f"parse url success {success}")
        if success:
            return data.get("data").get("taskId")
    
@retry(stop_max_attempt_number=3, wait_fixed=5000)
def queryNetSourceParse(task_id, dir_id,title):
    payload = {"action": "queryNetSourceParse", "version": "1.0", "taskId": task_id}
    url = "https://tw-efficiency.biz.aliyun.com/api/trans/queryNetSourceParse?c=tongyi-web"
    response = requests.post(url, headers=tongyi_headers, json=payload)
    results = []
    if response.ok:
        data = response.json().get("data")
        status = data.get("status")
        print(f"query source status {status}")
        if status == 0:
            urls = data.get("urls")
            for url in urls:
                results.append(
                    {
                        "fileId": url.get("fileId"),
                        "dirId": dir_id,
                        "fileSize": url.get("size"),
                        "tag": {
                            "fileType": "net_source",
                            "showName": title,
                            "lang": "cn",
                            "roleSplitNum": 0,
                            "translateSwitch": 0,
                            "transTargetValue": 0,
                            "client": "web",
                            "originalTag": "",
                        },
                    }
                )
            return results
        elif status == -1:
            time.sleep(1)
            return queryNetSourceParse(
                task_id=task_id, dir_id=dir_id,title=title
            )
        else:
            print(f"query source data = {data}")
            return None
        

@retry(stop_max_attempt_number=3, wait_fixed=5000)
def start(dir_id, files):
    payload = {
        "dirIdStr": dir_id,
        "files": files,
        "taskType": "net_source",
        "bizTerminal": "web",
    }
    url = "https://qianwen.biz.aliyun.com/assistant/api/record/blog/start?c=tongyi-web"
    r = requests.post(url, headers=tongyi_headers, json=payload)
    if r.ok :
        data = r.json()
        success = data.get("success")
        if success:
            genRecordIdList = data.get("data").get("genRecordIdList")
            if len(genRecordIdList)==1:
                return genRecordIdList[0]
            
@retry(stop_max_attempt_number=3, wait_fixed=5000)
def get_dir():
    """获取文件夹"""
    url = (
        "https://qianwen.biz.aliyun.com/assistant/api/record/dir/list/get?c=tongyi-web"
    )
    response = requests.post(url, headers=tongyi_headers)
    if response.ok:
        r = response.json()
        success = r.get("success")
        errorMsg = r.get("errorMsg")
        if success:
            return {x.get("dir").get("dirName"): x.get("dir").get("idStr") for x in r.get("data")}
        else:
            print(f"请求失败：{errorMsg}")
    else:
        print("请求失败：", response.status_code)


def main():
    generate_activation_code()
    refresh_token()
    dir_dict = get_dir()
    d = insert_podcast(dir_dict)
    episodes = get_history()
    eids = [x.get("eid") for x in episodes]
    progress = get_progress(eids)
    progress = {x.get("eid"): x for x in progress}
    for episode in episodes:
        if episode["eid"] in progress:
            episode["progress"] = progress.get(episode["eid"]).get("progress")
            episode["playedAt"] = progress.get(episode["eid"]).get("playedAt")
    insert_episode(episodes, d,dir_dict)
    update_month_data()

notion_helper = NotionHelper()

if __name__ == "__main__":
    main()



import calendar
from datetime import datetime
from datetime import timedelta
import hashlib
import os
import re
import requests
import base64
from podcast2notion.config import (
    RICH_TEXT,
    URL,
    RELATION,
    NUMBER,
    DATE,
    FILES,
    STATUS,
    TITLE,
    SELECT,
    CHECKBOX,
    MULTI_SELECT,
)
import pendulum

MAX_LENGTH = (
    1024  # NOTION 2000个字符限制https://developers.notion.com/reference/request-limits
)

tz = "Asia/Shanghai"


def get_heading(level, content):
    if level == 1:
        heading = "heading_1"
    elif level == 2:
        heading = "heading_2"
    else:
        heading = "heading_3"
    return {
        "type": heading,
        heading: {
            "rich_text": [
                {
                    "type": "text",
                    "text": {
                        "content": content[:MAX_LENGTH],
                    },
                }
            ],
            "color": "default",
            "is_toggleable": False,
        },
    }


def get_paragraph(content, bold=False):
    return {
        "type": "paragraph",
        "paragraph": {
            "rich_text": [
                {
                    "type": "text",
                    "text": {
                        "content": content[:MAX_LENGTH],
                    },
                    "annotations": {"bold": bold},
                }
            ],
            "color": "default",
        },
    }

def get_bulleted_list_item(content, bold=False):
    return {
        "type": "bulleted_list_item",
        "bulleted_list_item": {
            "rich_text": [
                {
                    "type": "text",
                    "text": {
                        "content": content[:MAX_LENGTH],
                    },
                    "annotations": {"bold": bold},
                }
            ],
            "color": "default",
        },
    }



def get_table_of_contents():
    """获取目录"""
    return {"type": "table_of_contents", "table_of_contents": {"color": "default"}}


def get_title(content):
    return {"title": [{"type": "text", "text": {"content": content[:MAX_LENGTH]}}]}


def get_rich_text(content):
    return {"rich_text": [{"type": "text", "text": {"content": content[:MAX_LENGTH]}}]}


def get_url(url):
    return {"url": url}


def get_file(url):
    return {"files": [{"type": "external", "name": "Cover", "external": {"url": url}}]}


def get_multi_select(names):
    return {"multi_select": [{"name": name} for name in names]}


def get_relation(ids):
    return {"relation": [{"id": id} for id in ids]}


def get_date(start, end=None):
    return {
        "date": {
            "start": start,
            "end": end,
            "time_zone": "Asia/Shanghai",
        }
    }


def get_icon(url):
    return {"type": "external", "external": {"url": url}}


def get_select(name):
    return {"select": {"name": name}}


def get_number(number):
    return {"number": number}


def get_quote(content):
    return {
        "type": "quote",
        "quote": {
            "rich_text": [
                {
                    "type": "text",
                    "text": {"content": content[:MAX_LENGTH]},
                }
            ],
            "color": "default",
        },
    }


def get_callout(content, icon):
    # 根据不同的划线样式设置不同的emoji 直线type=0 背景颜色是1 波浪线是2
    return {
        "type": "callout",
        "callout": {
            "rich_text": [
                {
                    "type": "text",
                    "text": {
                        "content": content[:MAX_LENGTH],
                    },
                }
            ],
            "icon":icon,
        },
    }


def get_rich_text_from_result(result, name):
    return result.get("properties").get(name).get("rich_text")[0].get("plain_text")


def get_number_from_result(result, name):
    return result.get("properties").get(name).get("number")


def format_time(time):
    """将秒格式化为 xx时xx分格式"""
    result = ""
    hour = time // 3600
    if hour > 0:
        result += f"{hour}时"
    minutes = time % 3600 // 60
    if minutes > 0:
        result += f"{minutes}分"
    return result


def format_date(date, format="%Y-%m-%d %H:%M:%S"):
    return date.strftime(format)


def timestamp_to_date(timestamp):
    """时间戳转化为date"""
    return datetime.utcfromtimestamp(timestamp) + timedelta(hours=8)


def get_first_and_last_day_of_month(date):
    # 获取给定日期所在月的第一天
    first_day = date.replace(day=1, hour=0, minute=0, second=0, microsecond=0)

    # 获取给定日期所在月的最后一天
    _, last_day_of_month = calendar.monthrange(date.year, date.month)
    last_day = date.replace(
        day=last_day_of_month, hour=0, minute=0, second=0, microsecond=0
    )+ timedelta(days=1)

    return first_day, last_day


def get_first_and_last_day_of_year(date):
    # 获取给定日期所在年的第一天
    first_day = date.replace(month=1, day=1, hour=0, minute=0, second=0, microsecond=0)

    # 获取给定日期所在年的最后一天
    last_day = date.replace(month=12, day=31, hour=0, minute=0, second=0, microsecond=0) + timedelta(days=1)

    return first_day, last_day


def get_first_and_last_day_of_week(date):
    # 获取给定日期所在周的第一天（星期一）
    first_day_of_week = (date - timedelta(days=date.weekday())).replace(
        hour=0, minute=0, second=0, microsecond=0
    )

    # 获取给定日期所在周的最后一天（星期日）
    last_day_of_week = first_day_of_week + timedelta(days=7)

    return first_day_of_week, last_day_of_week


def get_properties(dict1, dict2):
    properties = {}
    for key, value in dict1.items():
        type = dict2.get(key)
        if value == None:
            continue
        property = None
        if type == TITLE:
            property = {
                "title": [{"type": "text", "text": {"content": value[:MAX_LENGTH]}}]
            }
        elif type == RICH_TEXT:
            property = {
                "rich_text": [{"type": "text", "text": {"content": value[:MAX_LENGTH]}}]
            }
        elif type == NUMBER:
            property = {"number": value}
        elif type == STATUS:
            property = {"status": {"name": value}}
        elif type == FILES:
            property = {
                "files": [
                    {"type": "external", "name": "Cover", "external": {"url": value}}
                ]
            }
        elif type == DATE:
            property = {
                "date": {
                    "start": pendulum.from_timestamp(
                        value, tz="Asia/Shanghai"
                    ).to_datetime_string(),
                    "time_zone": "Asia/Shanghai",
                }
            }
        elif type == URL:
            property = {"url": value}
        elif type == CHECKBOX:
            property = {"checkbox": value}
        elif type == SELECT:
            property = {"select": {"name": value}}
        elif type == MULTI_SELECT:
            property = {"multi_select": [{"name": name} for name in value]}
        elif type == RELATION:
            property = {"relation": [{"id": id} for id in value]}
        if property:
            properties[key] = property
    return properties


def get_property_value(property):
    """从Property中获取值"""
    type = property.get("type")
    content = property.get(type)
    if content is None:
        return None
    if type == "title" or type == "rich_text":
        if len(content) > 0:
            return content[0].get("plain_text")
        else:
            return None
    elif type == "status" or type == "select":
        return content.get("name")
    elif type == "files":
        # 不考虑多文件情况
        if len(content) > 0 and content[0].get("type") == "external":
            return content[0].get("external").get("url")
        else:
            return None
    elif type == "date":
        return str_to_timestamp(content.get("start"))
    else:
        return content


def calculate_book_str_id(book_id):
    md5 = hashlib.md5()
    md5.update(book_id.encode("utf-8"))
    digest = md5.hexdigest()
    result = digest[0:3]
    code, transformed_ids = transform_id(book_id)
    result += code + "2" + digest[-2:]

    for i in range(len(transformed_ids)):
        hex_length_str = format(len(transformed_ids[i]), "x")
        if len(hex_length_str) == 1:
            hex_length_str = "0" + hex_length_str

        result += hex_length_str + transformed_ids[i]

        if i < len(transformed_ids) - 1:
            result += "g"

    if len(result) < 20:
        result += digest[0 : 20 - len(result)]
    md5 = hashlib.md5()
    md5.update(result.encode("utf-8"))
    result += md5.hexdigest()[0:3]
    return result


def transform_id(book_id):
    id_length = len(book_id)
    if re.match("^\d*$", book_id):
        ary = []
        for i in range(0, id_length, 9):
            ary.append(format(int(book_id[i : min(i + 9, id_length)]), "x"))
        return "3", ary

    result = ""
    for i in range(id_length):
        result += format(ord(book_id[i]), "x")
    return "4", [result]


def get_weread_url(book_id):
    return f"https://weread.qq.com/web/reader/{calculate_book_str_id(book_id)}"


def str_to_timestamp(date):
    if date == None:
        return 0
    dt = pendulum.parse(date)
    # 获取时间戳
    return int(dt.timestamp())


upload_url = "https://wereadassets.malinkang.com/"


def upload_image(folder_path, filename, file_path):
    # 将文件内容编码为Base64
    with open(file_path, "rb") as file:
        content_base64 = base64.b64encode(file.read()).decode("utf-8")

    # 构建请求的JSON数据
    data = {"file": content_base64, "filename": filename, "folder": folder_path}

    response = requests.post(upload_url, json=data)

    if response.status_code == 200:
        print("File uploaded successfully.")
        return response.text
    else:
        return None


def url_to_md5(url):
    # 创建一个md5哈希对象
    md5_hash = hashlib.md5()

    # 对URL进行编码，准备进行哈希处理
    # 默认使用utf-8编码
    encoded_url = url.encode("utf-8")

    # 更新哈希对象的状态
    md5_hash.update(encoded_url)

    # 获取十六进制的哈希表示
    hex_digest = md5_hash.hexdigest()

    return hex_digest


def download_image(url, save_dir="cover"):
    # 确保目录存在，如果不存在则创建
    if not os.path.exists(save_dir):
        os.makedirs(save_dir)

    file_name = url_to_md5(url) + ".jpg"
    save_path = os.path.join(save_dir, file_name)

    # 检查文件是否已经存在，如果存在则不进行下载
    if os.path.exists(save_path):
        print(f"File {file_name} already exists. Skipping download.")
        return save_path

    response = requests.get(url, stream=True)
    if response.status_code == 200:
        with open(save_path, "wb") as file:
            for chunk in response.iter_content(chunk_size=128):
                file.write(chunk)
        print(f"Image downloaded successfully to {save_path}")
    else:
        print(f"Failed to download image. Status code: {response.status_code}")
    return save_path


def upload_cover(url):
    cover_file = download_image(url)
    return upload_image("cover", f"{cover_file.split('/')[-1]}", cover_file)


def format_milliseconds(milliseconds):
    # Convert milliseconds to total seconds
    total_seconds = milliseconds // 1000

    # Calculate hours, minutes, and seconds
    hours = total_seconds // 3600
    minutes = (total_seconds % 3600) // 60
    seconds = total_seconds % 60

    # Format the output string based on the presence of hours
    if hours > 0:
        return f"{hours:02}:{minutes:02}:{seconds:02}"
    else:
        return f"{minutes:02}:{seconds:02}"

def get_embed(url):
    return {"type": "embed", "embed": {"url": url}}



from setuptools import setup, find_packages

setup(
    name="podcast2notion",
    version="0.2.2",
    packages=find_packages(),
    install_requires=[
        "requests",
        "pendulum",
        "retrying",
        "notion-client",
        "github-heatmap",
        "python-dotenv",
        "emoji",
    ],
    entry_points={
        "console_scripts": [
            "podcast = podcast2notion.podcast:main",
            "speech_text = podcast2notion.speech_text:main",
            "update_heatmap = podcast2notion.update_heatmap:main",
        ],
    },
    author="malinkang",
    author_email="linkang.ma@gmail.com",
    description="自动将习惯同步到Notion",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/malinkang/podcast2notionpro",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)



RICH_TEXT = "rich_text"
URL = "url"
RELATION = "relation"
NUMBER = "number"
DATE = "date"
FILES = "files"
STATUS = "status"
TITLE = "title"
SELECT = "select"
CHECKBOX = "checkbox"
MULTI_SELECT = "multi_select"
TZ = "Asia/Shanghai"
book_properties_type_dict = {
    "标题": TITLE,
    "Description": RICH_TEXT,
    "音频": RICH_TEXT,
    "Eid": RICH_TEXT,
    "链接": URL,
    "通义链接": URL,
    "发布时间": DATE,
    "时长": NUMBER,
    "时间戳": NUMBER,
    "状态": STATUS,
    "Podcast": RELATION,
    "喜欢": CHECKBOX,
    "日期": DATE,
    "收听进度": NUMBER,
}

TAG_ICON_URL = "https://www.notion.so/icons/hourglass_gray.svg"


movie_properties_type_dict = {
    "播客": TITLE,
    "Brief": RICH_TEXT,
    "Description": RICH_TEXT,
    "Pid": RICH_TEXT,
    "作者": RELATION,
    "全部": RELATION,
    "最后更新时间": DATE,
    "链接": URL,
    "收听时长": NUMBER,
    "通义链接": URL,
}
